# Ubuntu Continuous Integration Engine
# Copyright 2014 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License version 3, as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
# SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import urllib2

from ci_utils import data_store, ticket_states
from ci_libs import utils


class ImageObjectNotFound(Exception):
    """Raised when image object cannot be found."""
    def __init__(self, message):
        self.filename = message.split(",")[0].split(": ")[1]
        self.url = message.split("Error: Object GET failed: ")[1].split(
            " 404 Not Found")[0]

    def __str__(self):
        return "404 Not Found: {}".format(self.url)


def _get_image_artifact(ticket):
    artifact_base = utils.CI_URL + utils.TICKET_ARTIFACT_BASE
    try:
        url = artifact_base + '?type__exact=IMAGE&ticket__exact={}'.format(
            ticket)
        return utils.get(url)
    except urllib2.HTTPError, exc:
        if exc.code == 404 and ticket:
            sys.exit("Ticket number {} not found.".format(ticket))
        raise
    except urllib2.URLError, exc:
        raise


def _download(ticket, name, args_ticket):
    data = _get_image_artifact(ticket)
    ticket_id = str(ticket) + '-image'
    ds = data_store.create_for_ticket(ticket_id=ticket_id,
                                      auth_config=utils.AUTH_CONFIG,
                                      public=False)
    image = data['objects'][0][u'name']
    try:
        download = ds.get_file(image)
        fp = open(name, 'wb')
        fp.write(download)
        if args_ticket:
            print("Successfully downloaded image for ticket #{}.\n"
                  "Image file can be found at: {}".format(ticket, name))
        else:
            print("Successfully downloaded latest release candidate image.\n"
                  "Image file can be found at: {}".format(name))
    except data_store.DataStoreException as exc:
        if "404 Not Found" in exc.args[0]:
            raise ImageObjectNotFound(exc.message)
        raise
    except IOError:
        raise


def get_image(args):
    if args.ticket:
        ticket = args.ticket
    else:
        ticket_status_base = utils.CI_URL + utils.TICKET_STATUS_BASE
        url = ticket_status_base + '?current_workflow_step={}'.format(
            ticket_states.TicketWorkflowStep.COMPLETED.value)
        data = utils.get(url)
        try:
            ticket = max([x['id'] for x in data.get('objects', [])])
        except ValueError:
            sys.exit("No completed tickets found.")

    if os.path.exists(args.name):
        print("{} already exists.".format(args.name))
    elif not os.path.isdir(os.path.dirname(args.name)):
        os.makedirs(os.path.dirname(args.name))
        _download(ticket, args.name, args.ticket)
    else:
        _download(ticket, args.name, args.ticket)
